import typing

import torch.package
from loguru import logger

from EngineABC import EngineABC, ModelDescription
from to_wav import tensor2wav

if typing.TYPE_CHECKING:
    from .multi_v2_package import TTSModelMulti_v2
    from pathlib import Path


class Silero(EngineABC):
    def __init__(self, save_path: 'Path'):
        super().__init__(save_path)
        threads = 12
        device = torch.device('cpu')
        torch.set_num_threads(threads)
        dst = save_path / 'model_multi.pt'
        if not dst.exists():
            torch.hub.download_url_to_file(
                'https://models.silero.ai/models/tts/multi/v2_multi.pt',
                str(dst)
            )

        self.model: TTSModelMulti_v2 = torch.package.PackageImporter(dst).load_pickle("tts_models", "model")

        self.model.to(device)
        logger.debug(f"Loading speaker: v2_multi")
        self.sample_rate = 16000

    def synth(self, text: str, model: str, *args, **kwargs) -> bytes:
        # model is speaker
        return tensor2wav(
            self.model.apply_tts(texts=text, speakers=model, sample_rate=self.sample_rate)[0],
            self.sample_rate
        )

    def discovery(self) -> tuple[ModelDescription, ...]:
        return tuple(
            ModelDescription(
                engine=self.__class__.__name__,
                name=speaker_name,
                arguments=dict(),
                description=desc
            )
            for speaker_name, desc in {
                'aidar': 'ru',
                'baya': 'ru',
                'kseniya': 'ru',
                'irina': 'ru',
                'ruslan': 'ru',
                'natasha': 'ru',
                'thorsten': 'de',
                'tux': 'es',
                'gilles': 'fr',
                'lj': 'en',
                'dilyara': 'tt'
            }
        )
